/*
 * Copyright (c) 2010 Moreno Marzolla
 * Copyright (c) 2003-2005 The BISON Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package example.vman;

import peersim.config.*;
import peersim.core.*;
import peersim.util.*;

/**
 * This class prints statistics about the number of VMs running on each node.
 * It is assumed that the network nodes are instances of the
 * {@link example.loadbalance.BasicConsolidation} class.
 */
public class VMObserver implements Control {

    // ///////////////////////////////////////////////////////////////////////
    // Constants
    // ///////////////////////////////////////////////////////////////////////

    /**
     * The protocol to operate on.
     * 
     * @config
     */
    private static final String PAR_PROT = "protocol";

    // ///////////////////////////////////////////////////////////////////////
    // Fields
    // ///////////////////////////////////////////////////////////////////////

    /**
     * The name of this observer in the configuration file. Initialized by the
     * constructor parameter.
     */
    private final String name;

    /** Protocol identifier, obtained from config property {@link #PAR_PROT}. */
    private final int pid;

    // ///////////////////////////////////////////////////////////////////////
    // Constructor
    // ///////////////////////////////////////////////////////////////////////

    /**
     * Standard constructor that reads the configuration parameters. Invoked by
     * the simulation engine.
     * 
     * @param name
     *            the configuration prefix for this class.
     */
    public VMObserver(String name) {
        this.name = name;
        pid = Configuration.getPid(name + "." + PAR_PROT);
    }

    // ///////////////////////////////////////////////////////////////////////
    // Methods
    // ///////////////////////////////////////////////////////////////////////

    // Comment inherited from interface
    public boolean execute() {
        IncrementalFreq freqs = new IncrementalFreq();
        long time = peersim.core.CommonState.getTime();
	int capacity = 0;

        for (int i = 0; i < Network.size(); i++) {
            BasicConsolidation protocol = (BasicConsolidation) Network.get(i).getProtocol(
                    pid);
	    capacity = protocol.getCapacity();
            freqs.add((int)protocol.getValue());
        }

        /* Print statistics */
        // System.out.println(name + ": " + time + " " + freqs);
	System.out.print(name + ": " + time);
	for (int j=0; j<=capacity; ++j ) 
	    System.out.print(" " + freqs.getFreq(j));
	System.out.println();
        return false;
    }

}
