1; # not a function file

figure(1, "visible", "off");

## set (0,"Defaulttextfontsize", '26');
## text (1600, 1.5, "5 layers",'color', b); 

## Reads "filename", which contains output distilled from the output
## of peersim, and compute F(i,j) and Fzero(i).
##
## F(i,j) is the fraction of hosts with j-1 VMs at time i
## Fzero(i) is the maximum number of empty servers at time i if VM allocation were done optimally
function [F Fzero] = process_data( filename )
  a = load(filename); # load (filtered) output produced by peersim
  a(:,1) = []; # drop first column (timestamp)
  if (rows(a)>20)
    ## multiple runs. Quick & dirty way to average across all runs.
    aa = zeros(20, columns(a));
    ii = 1;
    for i=1:rows(a)/20
      aa += a(ii:(ii+19),:);
      ii += 20;
    endfor
    a = aa/20;
  endif

  capacity = columns(a)-1;
  ## a(i,j) is the number of servers with j-1 VMs at time i
  vm = sum(a .* repmat(0:capacity,rows(a),1),2); # vm(i) = number of running VMs at time i
  b = sum(a,2); # b(i) = number of servers at time i
  bb = repmat(b,1,columns(a));
  F = a./bb;
  Fzero = 1 .- (vm ./ capacity) ./ b; # optimal number of empty servers

  ##
  ## produce the area plot
  ##
  h = area(1:rows(F), F);
  ## F_0 is white

  set(h(1),"FaceColor",[1 1 1]);
  ## F_1 -> F_7 is light grey
  c = [.75 .75 .75];
  set(h(2:capacity+1),"FaceColor",c);
  set(h(2:capacity+1),"edgecolor",c);
  ## F_8 is dark grey
  set(h(capacity+1), "Facecolor", [.5 .5 .5]);
  
  ax = axis();
  ax(1) = 1;
  ax(4) = 1;
  axis(ax);
endfunction

set(gcf,"paperorientation","landscape");
papersize=[5 3.5] * 0.6; 
margin=[0 0];
set(gcf,"papersize",papersize);
set(gcf,"paperposition", [margin papersize-margin*2]);

##
## config-vman-static.txt
## Static system
##
hold on;
[F Fzero] = process_data("config-vman-static.out");

text(16, Fzero(16)-0.05, "F_0");
text(16, Fzero(16)+0.05, "F_8", "color","white");

plot(1:length(Fzero),Fzero,"k","linewidth",2);
legend("Optimal F_0","location","southeast");
xlabel("Time Step");
hold off;
print("config-vman-static.eps","-deps2","-color");

clf;

##
## config-vman-vmchurn.txt
## Static system with VM churn
##
hold on;
[F Fzero] = process_data("config-vman-vmchurn-high.out");

text(2, 0.1, "F_0");
text(2, 0.9, "F_8", "color","white");

plot(1:length(Fzero),Fzero,"k","linewidth",2);
legend("Optimal F_0","location","southeast");
xlabel("Time step");
ylabel("F_i");
hold off;
print("config-vman-vmchurn-high.eps","-deps2","-color");

clf;

##
## config-vman-dynamic.txt
##
hold on;
[F Fzero] = process_data("config-vman-dynamic.out");
plot(1:length(Fzero),Fzero,"k","linewidth",2);
legend("Optimal F_0", "location", "northeast");
line([5 5], [0 1]);
text(5.1,0.1,"-1000 servers");

line([10 10], [0 1]);
text(10.1,0.1,"+2000 servers");

text(2, 0.1, "F_0");
text(2, 0.9, "F_8", "color","white");
xlabel("Time step");
print("config-vman-dynamic.eps","-deps2","-color");
hold off;

clf;

##
## config-vman-stop.txt
## VMan stopped at step 5
##
hold on;
[F Fzero] = process_data("config-vman-stop.out");
plot(1:length(Fzero),Fzero,"k","linewidth",2);
legend("Optimal F_0", "location", "northeast");
line([5 5], [0 1]);
text(5.1,0.1,"V-Man stopped");

text(2, 0.1, "F_0");
text(2, 0.9, "F_8", "color","white");
xlabel("Time Step");
print("config-vman-stop.eps","-deps2","-color");
hold off;

clf;

##
## Comparison churn
##
[F0 Fzero0] = process_data("config-vman-static.out");
[F1 Fzero1] = process_data("config-vman-vmchurn-low.out");
[F2 Fzero2] = process_data("config-vman-vmchurn-high.out");
clf;
semilogy(1:length(Fzero0), Fzero0 - F0(:,1), ";{/Symbol D}_t = 0;", "linewidth", 2,
         1:length(Fzero1), Fzero1 - F1(:,1), ";{/Symbol D}_t in [-200, 200];", "linewidth", 2,
         1:length(Fzero2), Fzero2 - F2(:,1), ";{/Symbol D}_t in [-500, 500];", "linewidth", 2);
ax = axis();
ax(1) = 1;
axis(ax);
xlabel("Time Step");
ylabel("F_{0, opt} - F_0");
print("config-vman-vmchurn-compare.eps","-deps2");

clf;

##
## testk.txt
##
[F1 Fzero] = process_data("testk1.out");
[F2 Fzero] = process_data("testk2.out");
[F3 Fzero] = process_data("testk3.out");
clf;
semilogy(1:length(Fzero), Fzero - F3(:,1), ";K=20;", "linewidth", 2,
         1:length(Fzero), Fzero - F2(:,1), ";K=10;", "linewidth", 2,
	 1:length(Fzero), Fzero - F1(:,1), ";K=5;", "linewidth", 2);       
ax = axis();
ax(1) = 1;
axis(ax);
xlabel("Time Step");
ylabel("F_{0, opt} - F_0");
print("testk.eps","-deps2");
